'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Log = exports.defaultOptions = exports.indent = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.formatError = formatError;
exports.fromEnv = fromEnv;

var _chalk = require('chalk');

var _dateformat = require('dateformat');

var _dateformat2 = _interopRequireDefault(_dateformat);

var _lodash = require('lodash.defaults');

var _lodash2 = _interopRequireDefault(_lodash);

var _jestMessageUtil = require('jest-message-util');

var _path = require('path');

var _purdy = require('purdy');

var _stripAnsi = require('strip-ansi');

var _stripAnsi2 = _interopRequireDefault(_stripAnsi);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var indent = exports.indent = function indent(string, length) {
  var char = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : ' ';
  return '' + string.replace(/\n/g, '\n' + char.repeat(length));
};

function formatError(error) {
  var _separateMessageFromS = (0, _jestMessageUtil.separateMessageFromStack)(error.stack),
      message = _separateMessageFromS.message,
      stack = _separateMessageFromS.stack;

  var stackTrace = (0, _jestMessageUtil.formatStackTrace)(stack, { rootDir: (0, _path.dirname)((0, _path.dirname)(__dirname)), testMatch: [] }, { noStackTrace: false });
  return (0, _chalk.red)(message) + '\n' + stackTrace.replace(/\n {4}/g, '\n') + '\n';
}

function formatObject(object) {
  var str = (0, _purdy.stringify)(object, { indent: 2, arrayIndex: false });
  return (0, _chalk.grey)((0, _stripAnsi2.default)(str).length < 200 ? str.replace(/\n */g, ' ') : '\n' + str + '\n');
}

function format(value) {
  if (value instanceof Error) {
    return formatError(value);
  }
  return typeof value === 'string' ? value : formatObject(value);
}

var defaultOptions = exports.defaultOptions = {
  levels: ['emergency', 'alert', 'critical', 'error', 'warning', 'notice', 'info', 'debug'],
  colors: [_chalk.red.bold, _chalk.red, _chalk.red.bold, _chalk.red, _chalk.magenta, _chalk.yellow, _chalk.blue, _chalk.grey],
  inverse: [true, true],
  labels: ['EMERG', 'ALERT', 'CRITI', 'ERROR', ' WARN', ' NOTE', ' INFO', 'DEBUG'],
  separator: ' • ',
  stream: function stream(level) {
    return level <= 4 ? process.stderr : process.stdout;
  },
  format: format,
  displayTime: true,
  displayTimeFormat: 'yyyy-mm-dd HH:MM:ss.l',
  level: 'info'
};

var getStream = function getStream(level, _ref) {
  var stream = _ref.stream;
  return typeof stream === 'function' ? stream(level) : stream;
};

var Log = exports.Log = function () {
  function Log() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Log);

    this.options = (0, _lodash2.default)({}, options, defaultOptions);
    this.options.levels.forEach(function (name, logLevel) {
      _this[name] = function () {
        for (var _len = arguments.length, parts = Array(_len), _key = 0; _key < _len; _key++) {
          parts[_key] = arguments[_key];
        }

        return _this.write(logLevel, parts);
      };
    });
  }

  _createClass(Log, [{
    key: 'buildHeader',
    value: function buildHeader(level) {
      var _options = this.options,
          labels = _options.labels,
          colors = _options.colors,
          inverse = _options.inverse,
          displayTime = _options.displayTime,
          displayTimeFormat = _options.displayTimeFormat,
          separator = _options.separator;

      var time = displayTime ? (0, _chalk.grey)((0, _dateformat2.default)(Date.now(), displayTimeFormat)) : '';
      var color = colors[level];
      var labelColor = inverse[level] ? color.inverse : color;
      var label = '' + labels[level];
      return '' + (time && (0, _chalk.grey)('' + time + separator)) + labelColor(label) + color(separator);
    }
  }, {
    key: 'buildBody',
    value: function buildBody(parts) {
      var messages = parts.map(this.options.format);
      return messages.join(messages.some(function (s) {
        return s.includes('\n');
      }) ? '\n' : ' ');
    }
  }, {
    key: 'buildMessage',
    value: function buildMessage(level, parts) {
      var header = this.buildHeader(level);
      var body = this.buildBody(parts);
      return '' + header + indent(body, (0, _stripAnsi2.default)(header).length) + '\n';
    }
  }, {
    key: 'write',
    value: function write(level, parts) {
      if (level > this.level) {
        return;
      }
      getStream(level, this.options).write(this.buildMessage(level, parts));
    }
  }, {
    key: 'level',
    get: function get() {
      return this.options.levels.indexOf(this.options.level);
    }
  }]);

  return Log;
}();

function fromEnv() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var env = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : process.env || {};
  var environment = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : env.NODE_ENV || 'development';

  var mergedOptions = (0, _lodash2.default)(options, {
    level: env.LOG || env.LOG_LEVEL,
    displayTime: env.LOG_TIME,
    displayTimeFormat: env.LOG_TIME_FORMAT
  }, {
    development: { displayTime: false },
    test: { level: 'critical', displayTime: false }
  }[environment]);
  return new Log(mergedOptions);
}